#!/bin/bash
#
# Abilis Collector Uninstaller
#
# Author: Dario Fiumicello <fiumicello@antek.it>
# Ver: 2.1.0-1
#
# Copyright (c) 2013 - Abilis
#

FILES_TO_UNINSTALL="/opt/abilis/bin/collector /etc/opt/abilis/collector_init /opt/abilis/share/collector"
CONF_FILES="/etc/opt/abilis/collector.conf"
LOG_DIR="/var/opt/abilis/log/collector"

SED=`which sed` || (echo "FATAL: sed not found!"; exit 1)
PIDOF=`which pidof` || (echo "FATAL: pidof not found!"; exit 1)
RM=`which rm` || (echo "FATAL: rm not found!"; exit 1)
AWK=`which awk` || (echo "FATAL: awk not found!"; exit 1)

# Obtain database info from the CONFFILE
MYSQLHOST=`grep MYSQLHOST ${CONF_FILES} | ${AWK} -F ':' '{print $2}'`
MYSQLPORT=`grep MYSQLPORT ${CONF_FILES} | ${AWK} -F ':' '{print $2}'`
MYSQLDB=`grep MYSQLDB ${CONF_FILES} | ${AWK} -F ':' '{print $2}'`
MYSQLUSER=`grep MYSQLUSER ${CONF_FILES} | ${AWK} -F ':' '{print $2}'`
MYSQLPASS=`grep MYSQLPASS ${CONF_FILES} | ${AWK} -F ':' '{print $2}'`

if [[ ! -z "${MYSQLPASS}" ]]; then
  MYSQLPASS="-p${MYSQLPASS}"
fi

MYSQL_CMD="mysql ${MYSQLDB} -u${MYSQLUSER} ${MYSQLPASS}"

### FUNCTIONS DEFINITIONS ###

#
# function ask_user
#
# Ask a question to the user and force him to choose between a set of possible
# answers.
#
# Params:
# 1) The question to prompt to the user
# 2) A string with possible answers separated by a space
# 3) A variable which will contain the answer
#
# Returns always 0
#
function ask_user {

  if [[ $# != 3 ]]; then
    echo "ask_user: FATAL! 3 argouments are needed"
    exit 255
  fi

  local question=${1}
  local answers=${2}
  local result=${3}

  regex=`echo ${answers} | ${SED} 's/ /|/g'`

  local user_answer
  while [[ true ]]; do
    echo -en "${question} "
    read user_answer
    if [[ "${user_answer}" =~ ${regex} ]]; then
      eval $result="'${user_answer}'";
      return 0
    else
      echo "Please choose among \"$answers\""
    fi
  done
}

#
# func check_return_code
#
# Check if a program exitcode is equal or different than 0. If it is equal
# prints "[OK]\n", otherwise prints "[FAIL]\n" followed by an error message
# specified by the user
#
# Params:
# 1) The exitcode to check
# 2) (optional) The message to print in case of error
#
# Returns the passed exitcode
#
function check_return_code {
  if [[ $# -gt 2 ]]; then
    echo "check_return_code: FATAL! no more than 2 argouments are accepted"
    exit 255
  fi

  local code=${1}
  local errmsg=${2}

  if [[ ${code} == "0" ]]; then
    echo "[OK]"
  else
    echo -e "[FAIL]\n$errmsg"
  fi

  return ${code}
}

### END FUNCTIONS DEFINITIONS ###


### MAIN PROGRAM ###

echo
echo "################################"
echo "#    Abilis Linux Collector    #"
echo "#         Uninstaller          #"
echo "#             ---              #"
echo "#    (c) 2013 - abilis.net     #"
echo "################################"
echo

# Check if the install script is running as root
if [[ ${UID} != "0" ]]; then
  echo -e "ERROR: You need to be root to install the collector\n"
  exit 1
fi

# Check if a collector is still running
${PIDOF} collector > /dev/null
if [[ $? == "0" ]]; then
  echo "ERROR: A collector is already running!"
  echo -e "Please stop it before installing the new collector.\n"
  exit 1
fi

echo "According to ${CONF_FILES} these are your SQL credentials:"
echo "  MySQL host:  ${MYSQLHOST}"
echo "  MySQL user:  ${MYSQLUSER}"
echo "  MySQL db:    ${MYSQLDB}"
echo 
echo -n "WARNING! If you answer 'y' to the next question "
echo "you will PERMANENTLY DELETE ALL COLLECTED DATA!" 
ask_user "Do you want to delete the database? (y/n)" "y n" remove_db
echo
echo "Found the following configuration file: ${CONF_FILES}"
ask_user "Do you want to remove it? (y/n)" "y n" remove_conf
echo
echo "Found the following log directory: ${LOG_DIR}"
ask_user "Do you want to remove it? (y/n)" "y n" remove_log
echo
echo "**** Collector Uninstallation Summary ***"
echo
echo -n "The following files and directory will be removed:"
echo " ${FILES_TO_UNINSTALL}" | ${SED} 's@ @\n  @g'
echo
echo -n "Configuration file ${CONF_FILES} will "
if [[ "${remove_conf}" == "n" ]]; then echo -n "NOT "; fi
echo "be removed!"
echo -n "Log directory ${LOG_DIR} will "
if [[ "${remove_log}" == "n" ]]; then echo -n "NOT "; fi
echo "be removed!"
echo -en "Database '${MYSQLDB}' will "
if [[ "${remove_db}" == "n" ]]; then echo -n "NOT "; fi
echo "be removed!"
echo
ask_user \
  "Do you want to continue uninstallation? (yes/no)" \
  "yes no" \
  confirm_uninstall

if [[ "${confirm_uninstall}" == "no" ]]; then
  echo -e "\nUninstall aborted!\n"
  exit 1
fi

echo
echo "*** Begin uninstallation ***"

if [[ "${remove_conf}" == "y" ]]; then
  for file in ${CONF_FILES}; do
    echo -e "\nRemoving config file ${file}"
    ${RM} -rf ${file}
  done
fi
if [[ "${remove_log}" == "y" ]]; then 
  echo -e "\nRemoving log dir ${LOG_DIR}"
  ${RM} -rf ${LOG_DIR}
fi
if [[ "${remove_db}" == "y" ]]; then 
  echo -e "\nRemoving database '${MYSQLDB}'"
  ${MYSQL_CMD} -e "drop database ${MYSQLDB}"
fi

# Check if we are on a Debian-based or Redhat-based system and eventually
# remove them
UPDATE_RCD=`which update-rc.d 2>/dev/null` # Debian-based systems use update-rc.d
CHKCONFIG=`which chkconfig 2>/dev/null` # Redhat-based systems use chkconfig

if [[ -x "${UPDATE_RCD}" ]]; then
  echo -e "\nSystem is Debian-based, using update-rc.d for boot files uninstallation"
  ${UPDATE_RCD} -f collector remove > /dev/null 2>&1

elif [[ -x "${CHKCONFIG}" ]]; then
  echo -e "\nSystem is an RedHat-based, using chkconfig for boot files uninstallation"
  ${CHKCONFIG} --del collector > /dev/null 2>&1
else
  echo -e "\nWARNING: Unable to remove boot files by hand!"
  echo "You should remove them manually."
fi

echo
for file in ${FILES_TO_UNINSTALL}; do
  echo "Removing ${file}"
  ${RM} -rf ${file}
done
echo
echo "*** Collector uninstallation complete! ***"
echo

### MAIN PROGRAM END ###
